// DataManager.h: interface for the CDataManager class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(DATAMANAGER__INCLUDED_)
#define DATAMANAGER__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BaseObject.h"
#include "PackFile.h"
#include "DataFile.h"

	
namespace FileSystem
{
	class CCompressFile;

//!  Pak Element struct. 
/*!
	This struct if the definition of a element of a linked list
	used by the CPakFile class.
	\ingroup Base
	\sa CPakFile
*/
struct TPacksListElement
{
	//!A Pack file.
	CPackFile pack;

	//!next element
	TPacksListElement* next;

	//!prior element
	TPacksListElement* prior;
};


//!  Data managing class.
/*!
	This class it used for managing compress/uncompress data.
	\sa CPackFile
	\ingroup Base
*/
class CDataManager : public CBaseObject  
{
	friend CCompressFile;
public:

	//! CBaseObject class it's the parent of CDataManager class
	/*!
	    CBaseObject class it's the parent of CDataManager class
	*/
	typedef CBaseObject Inherit;


	//!  Finalize an CDataManager Object. 
	/*!
		Finalize an CDataManager Object.
	*/	
	CResult finalize();	
	
	//!  Initialize an CDataManager Object. 
	/*!
		initialize an CDataManager Object.
		\param newDataManagerPath = indicate the path of data files
		\retval CResult value indicate if the operation was correct
	*/		
	CResult initialize(char *newDataManagerPath);

	//!  Default constructor. 
	/*!
		Called before the object creation.
	*/		
	inline CDataManager(){Inherit();};

	//!  Default destructor. 
	/*!
		Called before the object destruction.
	*/
	virtual ~CDataManager(){};

public:

	//! Open a DataFile object
	/*!
	    This function opens a return a DataFile, the object
		could be either a Physicall or CompressFile, but
		using DataFile object keep the code clean of
		compresed or uncompresed data.

		You must allways call the closeFile function after
		using the file.

        \param filePath = Relative path to the file
		\retval a CDataFile object pointer
        \sa closeFile
	*/
	CDataFile* openFile ( const char* filePath);

	//! Close a opened file
	/*!
		This method close a opened file.

	    \param filePointer = a CDataFile pointer object
		\retval a CResult Objet indicate if the operation was valid.
		\sa openFile
	*/
	CResult closeFile ( CDataFile* filePointer);
	

	//! uncompress a file. 
	/*!
		uncompress a input file to output file giving a block size and total 
		size.

		\param inputFileName = input file name
		\param outputFileName = output file name
		\param blockSize = block size
		\param total = total compress size
		\retval CResult value indicate if the operation was correct
	*/	
	CResult uncompressFile (char* inputFileName,char* outputFileName,
		unsigned long int blockSize, unsigned long int total);

	//! compress a file. 
	/*!
		compress a input file to output file giving a block size, 
		store total size.

		\param inputFileName = input file name
		\param outputFileName = output file name
		\param blockSize = block size
		\param total = fille after call with the compres size of the data
		\retval CResult value indicate if the operation was correct
	*/	
	CResult compressFile (char* inputFileName,char* outputFileName,
		unsigned long int blockSize, unsigned long int* total);
	
	//! uncompress a file. 
	/*!
		uncompress a input file to output file giving a block size and total
		size.

		\param fileInputStream = input stream
		\param fileOutputStream = output stream
		\param blockSize = block size
		\param total = total compress size
		\retval CResult value indicate if the operation was correct
	*/	
	CResult uncompressFile (FILE* fileInputStream,FILE* fileOutputStream,
		unsigned long int blockSize, unsigned long int total);

	//! compress a file. 
	/*!
		compress a input file to output file giving a block size, store total
		size.

		\param fileInputStream = input stram
		\param fileOutputStream = output stream
		\param blockSize = block size
		\param total = fille after call with the compres size of the data
		\retval CResult value indicate if the operation was correct
	*/	
	CResult compressFile (FILE* fileInputStream,FILE* fileOutputStream,
		unsigned long int blockSize, unsigned long int* total);

	
	//! get the first pack
	/*! 
		get the first pack file
	*/
	inline TPacksListElement* getFirstPack(){return firstFile;}

private:
	//! Data directory path.
	/*!
	    Data directory path.
	*/
	char dataManagerPath[MAX_PATH];

	//! First file element in the packs lists.
	/*!
	    First file element in the packs lists. Default value is NULL
		\sa TPacksListElement
	*/
	TPacksListElement* firstFile;

	//! Last file element in the packs lists.
	/*!
	    Last file element in the packs lists. Default value is NULL
		\sa TPacksListElement
	*/
	TPacksListElement* lastFile;

	//! Scan for the data packs elements
	/*!
	    This function scan for data packs in the same directory level that
		the data path directory, and add then to the data packs list.

		\retval a CResult object indicate if the operation was valid.
	*/
	CResult scanForPacks();

	//! Add a Pack file
	/*!
	    This function add a new pack file to the data pack list.
		\retval a CResult object indicate if the operation was valid.
	*/
	CResult addPackFile(char* fileName);

protected:
    //! Compress a block of data
    /*!
        Compress a block of data int an output buffer which must be at least
		0.1% larger than sourceLen plus 12 bytes.

		\param dest = destination buffer
		\param destLen = size of the destination buffer
		\param source = source buffer
		\param sourceLen = size of the source buffer
		\retval a CResult object indicate if the operation was valid
		\sa uncompressBlock

    */
	CResult compressBlock (unsigned char far *dest,unsigned long far *destLen,
		const unsigned char far *source, unsigned long  sourceLen);

    //! Uncompress a block of data
    /*!
        Uncompress a block of data int an output buffer that must be large 
		enough to hold the entire uncompressed data.

		(The size of the uncompressed data must have been saved previously)

		\param dest = destination buffer
		\param destLen = size of the destination buffer
		\param source = source buffer
		\param sourceLen = size of the source buffer
		\retval a CResult object indicate if the operation was valid
		\sa uncompressBlock

    */
	CResult uncompressBlock (unsigned char far *dest,unsigned long far *destLen,
		const unsigned char far *source, unsigned long  sourceLen);
};
}

extern FileSystem::CDataManager FS;

//! A file, its used for the no object oriented FileSystem
typedef FileSystem::CDataFile FSFILE;

//! Init the FileSystem.No object oriented FileSystem
/*
	Init the FileSystem. If you use the no object oriented FileSystem you must
	call this function once. The FileSystem itself ist end we your app its close.
 */
bool InitFS(char* path);

//! Open a file.No object oriented FileSystem
/*
	Open a file.
	\param name = FileNamre
	\retval = A file handle, NULL, if couldn't open it.
 */
FSFILE* fsopen(const char* name);

//! Close a file.No object oriented FileSystem
/*
	Close a file.
	\param file = a valid file handle.	
 */
void fsclose(FSFILE* file);

	
//!  Seek the file.No object oriented FileSystem
/*!
	With this method you could seek the file
	position insinde de data file.
	\param file = a valid file handle.
	\param amount = amount of bytes to seak		
	\param pos = starting pos SEEK_CUR=from current position,SEEK_END=from end of file,SEEK_SET=beginning of file.
*/
int fsseek(FSFILE* file,long amount,int pos);

//!  Read and amout of bytes.No object oriented FileSystem
/*!
	You should call this function to read an
	amount of bytes.

	\param file = a valid file handle.
	\param amount = amount of bytes to read.
	\param buffer = a pointer of a buffer to
	 be fille with the data		 
	 \retval amount of byte readed
*/
int fsread(long amount,char* buffer,FSFILE* file);

//!  Read a string into the buffer.No object oriented FileSystem
/*!
	You should call this function to read a
	string into a buffer

	\param file = a valid file handle.
	\param max = maximum string size.
	\param buffer = a pointer of a buffer to
	 be fille with the data		 
	 \retval the string itself or NULL if fail
*/
char* fsgets(char* buffer,int max,FSFILE* file);

//! Determinate if its the end of the file.No object oriented FileSystem
/*!
	You should call this function to know if
	you have reach the end of the file.
	\param file = a valid file handle.
	 \retval non zero if you reach the end
*/
int fseof(FSFILE* file);


#endif // !defined(DATAMANAGER__INCLUDED_)





















